@@ -7,6 +7,7 @@ from functools import reduce
 from sqlite3 import Connection, DatabaseError
 from typing import Optional, List
 
+import numpy as np
 import pandas as pd
 from pandas.io.sql import DatabaseError as PandasDatabaseError
 
@@ -19,11 +20,25 @@ logger = logging.getLogger(__name__)
 
 
 class RDYFile:
-    def __init__(self, path: str = ""):
+    def __init__(self, path: str = "", sync_method: str = None):
+        """
+
+        :param sync_method: Must be "timestamp", "device_time" or "gps_time", "timestamp" uses the timestamp when the
+        measurement started to adjust the timestamps (outputs nanoseconds), "device_time" transforms the time series to the
+        datetime (outputs datetime), "gps_time" uses the utc gps time if available (outputs datetime), if no gps data
+        is available it will fallback to the "device_time" method, "ntp_time" uses network time, if not available, it
+        will fallback to the "device_time" methode
+        :param path: Path to the Ridy file to be imported (can be .sqlite or .rdy)
+        """
         self.path = path
         self.name: Optional[str] = ""
         self.extension: Optional[str] = ""
 
+        if sync_method is not None and sync_method not in ["timestamp", "device_time", "gps_time", "ntp_time"]:
+            raise ValueError("synchronize argument must 'timestamp', 'device_time', 'gps_time' or 'ntp_time' not %s" % sync_method)
+
+        self.sync_method = sync_method
+
         self.db_con: Optional[Connection] = None
 
         # RDY File Infos
@@ -34,9 +49,11 @@ class RDYFile:
         self.rdy_info_height: Optional[float] = None
         self.rdy_info_weight: Optional[float] = None
 
-        self.t0: Optional[str] = None
+        self.t0: Optional[np.datetime64] = None
         self.timestamp_when_started: Optional[int] = None
         self.timestamp_when_stopped: Optional[int] = None
+        self.ntp_timestamp: Optional[int] = None
+        self.ntp_date_time: Optional[np.datetime64] = None
         self.device: Optional[Device] = None
 
         # Sensors
@@ -59,6 +76,10 @@ class RDYFile:
 
         if self.path:
             self.load_file(self.path)
+
+        if self.sync_method:
+            self._synchronize()
+
         pass
 
     def __iter__(self):
@@ -71,6 +92,48 @@ class RDYFile:
     def __repr__(self):
         return "%s" % self.name
 
+    def _synchronize(self):
+        if self.sync_method == "timestamp":
+            for m in self.measurements.values():
+                m.synchronize("timestamp", self.timestamp_when_started)
+        elif self.sync_method == "device_time":
+            if self.t0:
+                for m in self.measurements.values():
+                    m.synchronize("device_time", self.timestamp_when_started, self.t0)
+            else:
+                logger.warning("t0 is None, falling back to timestamp synchronization")
+                self.sync_method = "timestamp"
+                self._synchronize()
+        elif self.sync_method == "gps_time":
+            if len(self.measurements[GPSSeries]) > 0:
+                sync_timestamp = self.measurements[GPSSeries].time[0]
+                utc_sync_time = self.measurements[GPSSeries].utc_time[0]
+
+                for i, t in enumerate(self.measurements[GPSSeries].utc_time):  # The first utc_time value ending with 000 is a real GPS measurement
+                    if str(t)[-3:] == "000":
+                        utc_sync_time = t
+                        sync_timestamp = self.measurements[GPSSeries].time[i]
+                        break
+
+                sync_time = np.datetime64(int(utc_sync_time*1e6), "ns")
+                for m in self.measurements.values():
+                    m.synchronize("gps_time", sync_timestamp, sync_time)
+            else:
+                logger.warning("No GPS time recording, falling back to device_time synchronization")
+                self.sync_method = "device_time"
+                self._synchronize()
+        elif self.sync_method == "ntp_time":
+            if self.ntp_timestamp and self.ntp_date_time:
+                for m in self.measurements.values():
+                    m.synchronize("gps_time", self.ntp_timestamp, self.ntp_date_time)
+            else:
+                logger.warning("No ntp timestamp and datetime, falling back to device_time synchronization")
+                self.sync_method = "device_time"
+                self._synchronize()
+        else:
+            raise ValueError("sync_method must 'timestamp', 'device_time', 'gps_time' or 'ntp_time' not %s" % self.sync_method)
+        pass
+
     def load_file(self, path: str):
         logger.info("Loading file: %s" % path)
 
@@ -118,7 +181,7 @@ class RDYFile:
                 self.rdy_info_weight = None
 
             if 't0' in rdy:
-                self.t0 = datetime.fromisoformat(rdy['t0'])
+                self.t0 = np.datetime64(rdy['t0'])
             else:
                 self.t0 = None
                 logger.info("No t0 in file: %s" % self.name)
@@ -135,6 +198,18 @@ class RDYFile:
                 self.timestamp_when_stopped = None
                 logger.info("No timestamp_when_stopped in file: %s" % self.name)
 
+            if 'ntp_timestamp' in rdy:
+                self.ntp_timestamp = rdy['ntp_timestamp']
+            else:
+                self.ntp_timestamp = None
+                logger.info("No ntp_timestamp in file: %s" % self.name)
+
+            if 'ntp_date_time' in rdy:
+                self.ntp_date_time = np.datetime64(rdy['ntp_date_time'])
+            else:
+                self.ntp_date_time = None
+                logger.info("No ntp_date_time in file: %s" % self.name)
+
             if "device" in rdy:
                 self.device = Device(**rdy['device_info'])
             else:
@@ -243,18 +318,38 @@ class RDYFile:
 
             # Info
             if info is not None:
-                self.rdy_format_version = info['rdy_format_version'][0] if len(info['rdy_format_version']) > 0 else None
-                self.rdy_info_name = info['rdy_info_name'][0] if len(info['rdy_info_name']) > 0 else None
-                self.rdy_info_sex = info['rdy_info_sex'][0] if len(info['rdy_info_sex']) > 0 else None
-                self.rdy_info_age = info['rdy_info_age'][0] if len(info['rdy_info_age']) > 0 else None
-                self.rdy_info_height = info['rdy_info_height'][0] if len(info['rdy_info_height']) > 0 else None
-                self.rdy_info_weight = info['rdy_info_weight'][0] if len(info['rdy_info_weight']) > 0 else None
+                if 'rdy_format_version' in info and len(info['rdy_format_version']) > 0:
+                    self.rdy_format_version = info['rdy_format_version'][0]
 
-                self.t0 = info['t0'][0] if len(info['t0']) > 0 else None
-                self.timestamp_when_started = info['timestamp_when_started'][0] if len(
-                    info['timestamp_when_started']) > 0 else None
-                self.timestamp_when_stopped = info['timestamp_when_stopped'][0] if len(
-                    info['timestamp_when_stopped']) > 0 else None
+                if 'rdy_info_name' in info and len(info['rdy_info_name']) > 0:
+                    self.rdy_info_name = info['rdy_info_name'][0]
+
+                if 'rdy_info_sex' in info and len(info['rdy_info_sex']) > 0:
+                    self.rdy_info_sex = info['rdy_info_sex'][0]
+
+                if 'rdy_info_age' in info and len(info['rdy_info_age']) > 0:
+                    self.rdy_info_age = info['rdy_info_age'][0]
+
+                if 'rdy_info_height' in info and len(info['rdy_info_height']) > 0:
+                    self.rdy_info_height = info['rdy_info_height'][0]
+
+                if 'rdy_info_weight' in info and len(info['rdy_info_weight']) > 0:
+                    self.rdy_info_weight = info['rdy_info_weight'][0]
+
+                if 't0' in info and len(info['t0']) > 0:
+                    self.t0 = np.datetime64(info['t0'][0])
+
+                if 'timestamp_when_started' and len(info['timestamp_when_started']) > 0:
+                    self.timestamp_when_started = info['timestamp_when_started'][0]
+
+                if 'timestamp_when_stopped' in info and len(info['timestamp_when_stopped']) > 0:
+                    self.timestamp_when_stopped = info['timestamp_when_stopped'][0]
+
+                if 'ntp_timestamp' in info and len(info['ntp_timestamp']) > 0:
+                    self.ntp_timestamp = info['ntp_timestamp'][0]
+
+                if 'ntp_date_time' in info and len(info['ntp_date_time']) > 0:
+                    self.ntp_date_time = np.datetime64(info['ntp_date_time'][0])
 
             # Measurements
             try:
@@ -342,6 +437,7 @@ class RDYFile:
                 logger.error(
                     "DatabaseError occurred when accessing subjective_comfort_measurements_table, file: %s" % self.name)
 
+            self.db_con.close()
         else:
             raise ValueError("File extension %s is not supported" % self.extension)
 
